/*********************************************************************
 *
 *                 BMP180 I2C Communication Protocol
 *
 *********************************************************************
 * FileName:        BMP180.c
 ********************************************************************/

/****** I N C L U D E S **********************************************************/


#include <xc.h>
#include <pic18f87k22.h>
#include <plib/i2c.h>
#include <plib/usart.h>
#include <math.h>
#include <stdio.h>
#include "BMP180.h"

#define _XTAL_FREQ 4000000

//Valores Calibracin sensor
short AC1; 
short AC2; 
short AC3; 
unsigned short AC4; 
unsigned short AC5; 
unsigned short AC6; 
short B1; 
short B2; 
short MB; 
short MC; 
short MD;
long B5;
short oss = 3; // Oversampling Setting (0,1,2,3 from ultra low power, to ultra hi-resolution)

//Variables para el clculo de temperatura y presin
long Temp_UT;
long True_Temp;
long Pres_UP;
long True_pres;
float Altitude;

//Variable datos a escribir
extern char TxdataBMP180[25];
extern char TxdataBMP180_LCD[16];

/**********************************************************************
 * Function:        Read_int_BMP180(unsigned char rom_code0, unsigned char rom_code1)
 * PreCondition:    None
 * Input:           Direcciones ROM a leer del sensor BMP180	
 * Output:          Retorna int del dispositivo	
 * Overview:		Se utiliza para leer los valores de calibracin del sensor BMP180	
 ***********************************************************************/
short Read_int_BMP180(unsigned char rom_code0, unsigned char rom_code1) {

    short Temp;
    char lsb, msb;
    
    //Leer msb
    IdleI2C();
    StartI2C();
    IdleI2C();
    WriteI2C(BMP180_ADDRESS_W); //sensor's address 0xB4
    IdleI2C();
    WriteI2C(rom_code0); //register temperature's address 0x07
    IdleI2C();
    RestartI2C();
    IdleI2C(); 
    WriteI2C(BMP180_ADDRESS_R); //sensor's address 0xB5
    msb = ReadI2C();
    IdleI2C();
    NotAckI2C();
    IdleI2C();
    StopI2C();
    
    //Leer lsb
    IdleI2C();
    StartI2C();
    IdleI2C();
    WriteI2C(BMP180_ADDRESS_W); //sensor's address 0xB4
    IdleI2C();
    WriteI2C(rom_code1); //register temperature's address 0x07
    IdleI2C();
    RestartI2C();
    IdleI2C();
    WriteI2C(BMP180_ADDRESS_R); //sensor's address 0xB5  
    lsb = ReadI2C();
    IdleI2C();
    NotAckI2C();
    IdleI2C();
    StopI2C();

    Temp = (((msb) << 8) + lsb); 
    
    return Temp;
    
}

/**********************************************************************
 * Function:        Read_Uint_BMP180(unsigned char rom_code0, unsigned char rom_code1)
 * PreCondition:    None
  * Input:          Direcciones ROM a leer del sensor BMP180	
 * Output:          Retorna unsigned int del dispositivo	
 * Overview:		Se utiliza para leer los valores de calibracin del sensor BMP180	
 ***********************************************************************/
unsigned short Read_Uint_BMP180(unsigned char rom_code0, unsigned char rom_code1) {

    unsigned short Temp;
    char lsb, msb;
    
    //Leer msb
    IdleI2C();
    StartI2C();
    IdleI2C();
    WriteI2C(BMP180_ADDRESS_W); //sensor's address 0xB4  
    IdleI2C();
    WriteI2C(rom_code0); //register temperature's address 0x07  
    IdleI2C();
    RestartI2C();
    IdleI2C();
    WriteI2C(BMP180_ADDRESS_R); //sensor's address 0xB5
    msb = ReadI2C();
    IdleI2C();
    NotAckI2C();
    IdleI2C();
    StopI2C();  
    //Leer lsb
    IdleI2C();
    StartI2C();
    IdleI2C();
    WriteI2C(BMP180_ADDRESS_W); //sensor's address 0xB4
    IdleI2C();
    WriteI2C(rom_code1); //register temperature's address 0x07
    IdleI2C();
    RestartI2C();
    IdleI2C();
    WriteI2C(BMP180_ADDRESS_R); //sensor's address 0xB5  
    lsb = ReadI2C();
    IdleI2C();
    NotAckI2C();
    IdleI2C();
    StopI2C();

    Temp = (((msb) << 8) + lsb);
    
    return Temp;

}

/**********************************************************************
 * Function:        Read_byte_BMP180(unsigned char rom_code0)
 * PreCondition:    None
 * Input:           Direccin ROM a leer del sensor BMP180	
 * Output:          Retorna Byte leido del sensor BMP180	
 * Overview:		Se utiliza para la lectura de un byte del sensor BMP180	
 ***********************************************************************/
char Read_byte_BMP180(unsigned char rom_code0) {
 
    char Temp;
    
    //Leer msb
    IdleI2C();
    StartI2C();
    IdleI2C();
    WriteI2C(BMP180_ADDRESS_W); //sensor's address 0xB4
    IdleI2C();
    WriteI2C(rom_code0); //register temperature's address 0x07
    IdleI2C();
    RestartI2C();
    IdleI2C();
    WriteI2C(BMP180_ADDRESS_R); //sensor's address 0xB5
  
    Temp = ReadI2C();
    IdleI2C();
    NotAckI2C();
    IdleI2C();
    StopI2C();
  
    return Temp;

}

/**********************************************************************
 * Function:        Write_BMP180(unsigned char addr, unsigned char data)
 * PreCondition:    Direccin ROM y datos a escribir
 * Input:           None	
 * Output:          None	
 * Overview:		Se utiliza para la escritura en la rom del sensor BMP180	
 ***********************************************************************/
void Write_BMP180(unsigned char addr, unsigned char data){
    
    IdleI2C();
    StartI2C();
    IdleI2C();
    WriteI2C(BMP180_ADDRESS_W); //Direccin del sensor BMP180
    IdleI2C();
    WriteI2C(addr); //Registro al que accedo
    WriteI2C(data); //Dato a grabar
    IdleI2C();
    NotAckI2C();
    IdleI2C();
    StopI2C();

}

/**********************************************************************
 * Function:        Read_UT_BMP180(void)
 * PreCondition:    None
 * Input:           None	
 * Output:          long con datos de temperatura	
 * Overview:		Lectura de la temeperatura no compensada del sensor BMP180	
 ***********************************************************************/
long Read_UT_BMP180(void){
    
    long UT;
    //Write 0x2E into reg 0xF4
    Write_BMP180(0xF4,0x2E);
    //wait 4.5ms
    __delay_ms(5);
    
    UT = Read_int_BMP180(0xF6,0xF7);
    
    return UT;
    
}

/**********************************************************************
 * Function:        Get_True_Temp(long UT)
 * PreCondition:    Lectura de la temperatura no compensada y datos de calibracin del sensor BMP180
 * Input:           Temperatura no compensada	
 * Output:          Temperatura real	
 * Overview:		Calcula la temperatura real	
 ***********************************************************************/


long Get_True_Temp(long UT){
    
    long X1,X2 = 0;
    long T = 0;
    X1 = (UT- (long)AC6)* ((long)AC5)/32768;
    X2 = ( (long)MC*2048)/(X1+MD);
    B5 = X1+X2;
    T = (B5+8)/16;
    return T;
}
/**********************************************************************
 * Function:        Read_UP_BMP180(short OSS)
 * PreCondition:    None
 * Input:           Resolucin del sensor OSS (0, 1, 2, 3)	
 * Output:          Presin atmosfrica no compensada	
 * Overview:		Lectura de la presin atmosfrica no compensada	
 ***********************************************************************/
long Read_UP_BMP180(short OSS){
    
    long UP = 0;
    unsigned char msb, lsb, xlsb;
    // Write 0x34+(OSS<<6) into register 0xF4 
    Write_BMP180(0xF4,0x34+(OSS<<6));
    //Dependiendo de la resolucin escogida OSS habra un tiempo de conversin diferente
    //Delay OSS= 0 4.5ms; 1 7.5ms; 2 13.5ms; 3 25.5ms
    switch(OSS){
        case 0:
            __delay_ms(5);
            break;
        case 1:
            __delay_ms(8);
            break;
        case 2:
            __delay_ms(14);
            break;
        case 3:
            __delay_ms(26);
            break;     
    }
    
    msb = Read_byte_BMP180(0xF6);
    lsb = Read_byte_BMP180(0xF7);
    xlsb = Read_byte_BMP180(0xF8);
    
    UP = (long)((((long)msb << 16) | ((long) lsb << 8) |(long) xlsb) >> (8 - OSS));
    return UP;
    
}

/**********************************************************************
 * Function:        Get_True_Pres(long up)
 * PreCondition:    Lectura de la presin atmoefrica no compensada
 * Input:           Presin atmosfrica no compensada y datos de calibracin del sensor BMP180	
 * Output:          Presin atmosferica real	
 * Overview:		Calcula la presin atmosfrica real	
 ***********************************************************************/
long Get_True_Pres(long up)
{
 	long pressure, x1, x2, x3, b3, b6 = 0;
	unsigned long b4, b7 = 0;

    b6 = B5 - 4000;
    /*****calculate B3************/
    x1 = (b6 * b6) >> 12;
    x1 *= B2;
    x1 >>= 11;

    x2 = (AC2 * b6);
    x2 >>= 11;

    x3 = x1 + x2;

    b3 = ((((long) AC1 * 4 + x3) << oss) + 2) / 4;

    //*****calculate B4************
    x1 = (AC3 * b6) >> 13;
    x2 = (B1 * ((b6 * b6) >> 12)) >> 16;
    x3 = ((x1 + x2) + 2) >> 2;
    b4 = (AC4 * (unsigned long) (x3 + 32768)) >> 15;

    b7 = ((unsigned long) up - b3) * (50000 >> oss);
    if (b7 < 0x80000000) {
        pressure = (b7 << 1) / b4;
    } else {
        pressure = (b7 / b4) << 1;
    }

    x1 = (pressure >> 8);
    x1 *= x1;
    x1 = (x1 * 3038) / 65536;
    x2 = (-7357 * pressure);
    x2 >>= 16;
    pressure += (x1 + x2 + 3791) >> 4; // pressure in Pa

    return pressure;
   
}

/**********************************************************************
 * Function:        bmp085_Altitude(float pressure, float CurrentPressSeeLevel )
 * PreCondition:    Calculo presin atmesfrica real
 * Input:           Presin atmosfrica real y presin estimada al nivel del mar	
 * Output:          Altitud	
 * Overview:		Calcula la altitud a la que nos encontramos	
 ***********************************************************************/
float bmp085_Altitude(float pressure, float CurrentPressSeeLevel ){
    
   float A = ((float)pressure)/CurrentPressSeeLevel;
   float B = 1/5.255;
   float C = pow(A,B);
   C = 1 - C;
   C = C * 44330;

   return C;
}

void Get_CalValue_BMP180(void){

    AC1 = Read_int_BMP180(0xAA, 0xAB);
    AC2 = Read_int_BMP180(0xAC, 0xAD);
    AC3 = Read_int_BMP180(0xAE, 0xAF);
    AC4 = Read_Uint_BMP180(0xB0, 0xB1);
    AC5 = Read_Uint_BMP180(0xB2, 0xB3);
    AC6 = Read_Uint_BMP180(0xB4, 0xB5);
    B1 = Read_int_BMP180(0xB6, 0xB7);
    B2 = Read_int_BMP180(0xB8, 0xB9);
    MB = Read_int_BMP180(0xBA, 0xBB);
    MC = Read_int_BMP180(0xBC, 0xBD);
    MD = Read_int_BMP180(0xBE, 0xBF);
 
}

void Read_Dev_BMP180(void) {
    //Leo los valores Uncompensated Temperature, Pressure value
    float temp;
    Temp_UT = Read_UT_BMP180();
    Pres_UP = Read_UP_BMP180(oss);

    //Calculo la temperatura real con los valores de calibracin
    True_Temp = Get_True_Temp(Temp_UT);
    //Calcula la presin atmosfrica con los valores de calibracin y la resolucin OSS
    True_pres = Get_True_Pres(Pres_UP);
    //Calculo la altitud a la que me encuentro a partir de la presin obtenida y la presin a nivel del mar en ese momento
    Altitude = bmp085_Altitude(True_pres, 103160.0);
    //Temperatura con un decimal
    temp= ((float)True_Temp)/10;
    sprintf(TxdataBMP180, "Presion atmosferica: %ld Pa; Temperatura: %.1f%cC;", True_pres,temp,248); 
    sprintf(TxdataBMP180_LCD,"%ldPa; %.1f%cC", True_pres,temp,223);
}
