/********************************************************************
*
*
*
*********************************************************************
* FileName:         Delay.c
* Dependencies:     See INCLUDES section below
* Processor:        PIC18F87K22
* Compiler:         HTC
* Company:          Microchip Technology, Inc.
*
* Software License Agreement:
*
* The software supplied herewith by Microchip Technology Incorporated
* (the "Company") for its PICmicro® Microcontroller is intended and
* supplied to you, the Company's customer, for use solely and
* exclusively on Microchip PICmicro Microcontroller products. The
* software is owned by the Company and/or its supplier, and is
* protected under applicable copyright laws. All rights are reserved.
* Any use in violation of the foregoing restrictions may subject the
* user to criminal sanctions under applicable laws, as well as to
* civil liability for the breach of the terms and conditions of this
* license.
*
* THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
* WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
* TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
* PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
* IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
* CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
*********************************************************************
* File Description: This file contains functions related to delaying
*                   the system for 1 Sec, 1 mSec or 1uSec. The delay
*                   will automatically change depending on the current
*                   system frequency.
*
* Change History:
* Author               Date        Version
* Chris Tucker        5/2/11         1.1  (Initial Release)
********************************************************************/

/**
*   @file Delay.c
*   @brief
*   @defgroup
*
*/

#include        <htc.h>
#include        "delay.h"

#define LF                      0
#define MF                      1
#define HF                      2
#define PLL                     3
#define XT_10                   4
extern unsigned char Freq;

/********************************************************************
* Function:             Delay_Sec
*
* PreCondition: 	System Frequency
*
* Side Effects: 	Delay System for a secound
*/
/**
* @ingroup  		Delay
*
* @brief    		Thfs function will delay the system for a secound.
*
* @note    		Different delay values are used for each Frequency.
*/
/*******************************************************************/
void Delay_Sec(unsigned char numSec)
{
    unsigned int    cntdown;

    while (numSec--)
    {
        if (Freq == PLL)
        {   cntdown = PLL_1SEC; }        
        else if (Freq == HF)
        {   cntdown = HF_1SEC; }         
        else if (Freq == MF)
        {   cntdown = MF_1SEC; } 
        else if (Freq == XT_10)
        {   cntdown = XT_10MHZ_1SEC; }
        else
        {   cntdown = LF_1SEC; }           

            while (cntdown--)
            {
                DELAY3_250()
            }
    }
}

/********************************************************************
* Function:             Delay_mSec
*
* PreCondition: 	System Frequency
*
* Side Effects: 	Delay System for a millisecond
*/
/**
* @ingroup  		Delay
*
* @brief    		Thfs function will delay the system for a millisecond.
*
* @note    		Different delay values are used for each Frequency.
*/
/*******************************************************************/
void Delay_mSec(unsigned char numSec)
{
    unsigned int    cntdown;

    while (numSec--)
    {
        if (Freq == PLL)
        {   cntdown = PLL_1mSEC; }        
        else if (Freq == HF)
        {   cntdown = HF_1mSEC; }         
        else if (Freq == MF)
        {   cntdown = MF_1mSEC; }
        else if (Freq == XT_10)
        {   cntdown = XT_10MHZ_1mSEC; }
        else
        {   cntdown = LF_1mSEC; }           

            while (cntdown--)
            {
                DELAY3_1()
            }
    }
}

/********************************************************************
* Function:             Delay_uSec
*
* PreCondition: 	System Frequency
*
* Side Effects: 	Delay System for a microsecond
*/
/**
* @ingroup  		Delay
*
* @brief    		Thfs function will delay the system for a microsecond.
*
* @note    		Different delay values are used for each Frequency.
*/
/*******************************************************************/
void Delay_uSec(unsigned int numSec)
{
    unsigned int    cntdown;

    LF_SCALE_1uSEC();
    MF_SCALE_1uSEC();

    while(numSec--)
    {
        if (Freq == PLL)
        {   cntdown = PLL_1uSEC; }         
        else if (Freq == HF)
        {   cntdown = HF_1uSEC; }         
        else if (Freq == MF)
        {   cntdown = SCALED_1uSEC; }
        else if (Freq == XT_10)
        {   cntdown = XT_10MHZ_1uSEC; }
        else
        {   cntdown = SCALED_1uSEC; }        

            while (cntdown--)
            {
                _delay(1);
            }
    }
}

